#include <SoftwareSerial.h>
#include <EEPROM.h>

const byte rxPin = 10;
const byte txPin = 11;
// Set up a new SoftwareSerial object
SoftwareSerial HC05 (rxPin, txPin);

//Input/Output
int stepPin  = 2;
int dirPin  = 3;
int enPin = 4;
int hallPin = A0;
int buzzer = 6;

//Variables
bool go_up = false;
bool go_down = false;
float actual_steps;
float maximum_steps;

int stepDelay = 100;



void setup() {
  // Define pin modes for TX and RX
  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  pinMode(stepPin, OUTPUT);
  pinMode(dirPin, OUTPUT);
  pinMode(enPin, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(hallPin, INPUT);
  digitalWrite(enPin, HIGH);
  digitalWrite(buzzer, LOW);
    
  // Set the baud rate for the SoftwareSerial object
  HC05.begin(9600);
  Serial.begin(9600);  
  EEPROM.get(0, maximum_steps);
}

void loop() 
{
  if(go_up){
    Roll_up();
  }
  else if(go_down){
    Roll_down();
  }
  
  if(HC05.available()>0)
  {
    char byteRead = HC05.read();;     //Read data from the BT module  
    //Up
    if(byteRead == 'u'){
      if(!go_down){
        go_up = true;
        tone(buzzer, 1500, 200);
      }
    }
    
    //Down
    else if(byteRead == 'd'){
      if(!go_up){
        go_down = true;
        tone(buzzer, 1500, 200);
      }
    }

    //Get Home
    else if(byteRead == 'g'){
       Home();
       tone(buzzer, 1500, 600);
    }

    //Maximum value
    else if(byteRead == 'm'){
      Set_max();
      tone(buzzer, 1500, 200);
    }
    //Stop
    else if(byteRead == 's'){
      go_up = false;
      go_down = false;
      digitalWrite(enPin, HIGH);       //Disable driver
    }

    //Roll to the maximum
    if(byteRead == 'b'){
      Max_down();
      tone(buzzer, 1500, 600);
    }
    
    
        
  }
}//END void loop   

void Max_down(){
  while(actual_steps < maximum_steps){
      digitalWrite(enPin, LOW);       //Enable driver
      digitalWrite(dirPin, LOW);      //Rotate CCW
      digitalWrite(stepPin, HIGH);
      delayMicroseconds(stepDelay);
      digitalWrite(stepPin, LOW);
      delayMicroseconds(stepDelay);
      actual_steps++;                 //Increment steps
    }
}

void Roll_up(){
  if(!digitalRead(hallPin)){
    digitalWrite(enPin, LOW);       //Enable driver
    digitalWrite(dirPin, HIGH);     //Rotate CW
    digitalWrite(stepPin, HIGH);
    delayMicroseconds(stepDelay);
    digitalWrite(stepPin, LOW);
    delayMicroseconds(stepDelay);
    actual_steps--;   
  }
  else{
    digitalWrite(enPin, HIGH);
  }
  
  
}

void Roll_down(){
  if(EEPROM.read(4) == 111){
    if(actual_steps < maximum_steps){
      digitalWrite(enPin, LOW);       //Enable driver
      digitalWrite(dirPin, LOW);      //Rotate CCW
      digitalWrite(stepPin, HIGH);
      delayMicroseconds(stepDelay);
      digitalWrite(stepPin, LOW);
      delayMicroseconds(stepDelay);
      actual_steps++;                 //Increment steps
    }
  }
  else if(EEPROM.read(4) == 0){
    digitalWrite(enPin, LOW);       //Enable driver
    digitalWrite(dirPin, LOW);      //Rotate CCW
    digitalWrite(stepPin, HIGH);
    delayMicroseconds(stepDelay);
    digitalWrite(stepPin, LOW);
    delayMicroseconds(stepDelay);
    actual_steps++;                 //Increment steps
  }
  
  
}

void Home(){
  while(!digitalRead(hallPin)){
    digitalWrite(enPin, LOW);       //Enable driver
    digitalWrite(dirPin, HIGH);     //Rotate CW
    digitalWrite(stepPin, HIGH);
    delayMicroseconds(stepDelay);
    digitalWrite(stepPin, LOW);
    delayMicroseconds(stepDelay);
  }
  actual_steps = 0;
}

void Set_max(){
  EEPROM.put(0, actual_steps);
  if(EEPROM.read(4) == 111){
    EEPROM.write(4, 0); 
  }
  else{
    EEPROM.write(4, 111); 
  }
   
}
